@@ -1,9 +1,11 @@
 from django.utils import simplejson
+from django.http import QueryDict
+
 from dynamicresponse.response import *
 
 class DynamicFormatMiddleware:
     """
-    Provides support for dynamic content negotiation, both in request  and reponse.
+    Provides support for dynamic content negotiation, both in request and reponse.
     """
     
     def _flatten_dict(self, obj, prefix=''):
@@ -11,26 +13,41 @@ class DynamicFormatMiddleware:
         Converts a possibly nested dictionary to a flat dictionary.
         """
         
-        encoded_dict = {}
+        encoded_dict = QueryDict('').copy()
+        
         if hasattr(obj, 'items'):
             for key, value in obj.items():
-            
+                
                 item_key = '%(prefix)s%(key)s' % { 'prefix': prefix, 'key': key }
-            
-                # Decode lists into keys w/index for formset
+                
+                # Flatten lists for formsets and model choice fields
                 if isinstance(value, list):
                     for i, item in enumerate(value):
-                        item_prefix = '%(key)s-%(index)d-' % { 'key': key, 'index': i }
-                        encoded_dict.update(self._flatten_dict(item, prefix=item_prefix))
-                    
-                # Decode nested objects into their ID/PK
+                        
+                        if isinstance(item, dict):
+                            
+                            # Flatten nested object to work with formsets
+                            item_prefix = '%(key)s-%(index)d-' % { 'key': key, 'index': i }
+                            encoded_dict.update(self._flatten_dict(item, prefix=item_prefix))
+                            
+                            # ID for use with model multi choice fields
+                            id_value = item.get('id', None)
+                            if id_value:
+                                encoded_dict.update({ key: id_value })
+                            
+                        else:
+                            
+                            # Value for use with model multi choice fields
+                            encoded_dict.update({ key: item })
+                
+                # ID for use with model choice fields
                 elif isinstance(value, dict):
                     encoded_dict[item_key] = value.get('id', value)
-            
+                
                 # Other values are used directly
                 else:
                     encoded_dict[item_key] = unicode(value)
-                
+        
         return encoded_dict
         
     def process_request(self, request):
@@ -51,12 +68,10 @@ class DynamicFormatMiddleware:
                     # Replace request.POST with flattened dictionary from JSON
                     decoded_dict = simplejson.loads(request.raw_post_data)
                     request.POST = request.POST.copy()
-                    request.POST.clear()
-                    for key, val in self._flatten_dict(decoded_dict).items():
-                        request.POST[key] = val
+                    request.POST = self._flatten_dict(decoded_dict)
                 except:
                     return HttpResponse('Invalid JSON', status=400)
-            
+    
     def process_response(self, request, response):
         """
         Handles rendering dynamic responses.
